import { uint32 } from "./number";
/**
 * Concatenable and sliceable types.
 * Native types Array and String implements this interface.
 */
export interface Concatenable<E extends Concatenable<E>> {
    /**
     * Number of elements.
     */
    readonly length: uint32;
    /**
     * "abc".slice(0, 3) === "abc"
     * "abc".slice(1, 2) === "b"
     *
     * @param lower index where the slice starts.
     * @param excludedUpper excluded index where the slice ends.
     * @return Slice between indexes [`lower', `excludedUpper'[.
     */
    readonly slice: (lower: uint32, excludedUpper: uint32) => E;
    /**
     * @param other appended elements
     * @return `other' appended to `this'.
     */
    readonly concat: (other: E) => E;
}
/**
 * Minimal implementation of Concatenable.
 * This enables to model a number of elements without their availability.
 *
 * For instance ConcatenableLength(3) can represent "abc", [1, 2, 3], or any
 * 3-elements list.
 */
export declare class ConcatenableLength implements Concatenable<ConcatenableLength> {
    readonly length: uint32;
    /** Nominal typing */
    private readonly brandConcatenableLength;
    /**
     * @param length Concatenable#length
     */
    constructor(length: uint32);
    /** @override */
    slice(lower: uint32, excludedUpper: uint32): ConcatenableLength;
    /** @override */
    concat(other: ConcatenableLength): ConcatenableLength;
}
