/*
    Copyright (C) 2018  Victorien Elvinger

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
import { assert } from "./assert";
import { isUint32 } from "./number";
/**
 * Minimal implementation of Concatenable.
 * This enables to model a number of elements without their availability.
 *
 * For instance ConcatenableLength(3) can represent "abc", [1, 2, 3], or any
 * 3-elements list.
 */
var ConcatenableLength = /** @class */ (function () {
    /**
     * @param length Concatenable#length
     */
    function ConcatenableLength(length) {
        this.length = length;
        assert(function () { return isUint32(length); }, "length ∈ uint32");
    }
    // Access
    /** @override */
    ConcatenableLength.prototype.slice = function (lower, excludedUpper) {
        var _this = this;
        assert(function () { return isUint32(lower); }, "lower ∈ uint32");
        assert(function () { return lower < _this.length; }, "lower < this.length");
        assert(function () { return isUint32(excludedUpper); }, "excludedUpper ∈ uint32");
        assert(function () { return excludedUpper <= _this.length; }, "excludedUpper <= this.length");
        assert(function () { return lower <= excludedUpper; }, "lower <= excludedUpper");
        return new ConcatenableLength(excludedUpper - lower);
    };
    /** @override */
    ConcatenableLength.prototype.concat = function (other) {
        var _this = this;
        assert(function () { return isUint32(_this.length + other.length); }, "valid concatenation");
        return new ConcatenableLength(this.length + other.length);
    };
    return ConcatenableLength;
}());
export { ConcatenableLength };
//# sourceMappingURL=concatenable.js.map