import { uint32 } from "./number";
/**
 * Possible order relation between two intervals.
 *
 * @example
 * [0, 2] < [4, 6]
 * [0, 2] <: [3, 6]
 * [0, 4] <∩ [4, 6] (because their intersection is not empty)
 * [0, 2] ⊂ [0, 3]
 * [0, 2] = [0, 2]
 */
export declare const enum IntervalOrdering {
    BEFORE = -4,
    PREPENDABLE = -3,
    OVERLAPPING_BEFORE = -2,
    INCLUDED_BY = -1,
    EQUAL = 0,
    INCLUDING = 1,
    OVERLAPPING_AFTER = 2,
    APPENDABLE = 3,
    AFTER = 4,
}
/**
 * Interval of uint32.
 */
export declare class IntInterval {
    /**
     * @param lower {@link IntInterval#lower }
     * @param length  {@link IntInterval#length }
     */
    protected constructor(lower: uint32, length: uint32);
    /**
     * @param lower {@link IntInterval#lower }
     * @param length {@link IntInterval#length }
     * @return New interval.
     */
    static fromLength(lower: uint32, length: uint32): IntInterval;
    /**
     * @param lower {@link IntInterval#lower }
     * @param upper {@link IntInterval#length }
     * @return New interval.
     */
    static fromBounds(lower: uint32, upper: uint32): IntInterval;
    /**
     * Lower bound.
     */
    readonly lower: uint32;
    /**
     * Number of elements in the interval.
     */
    readonly length: uint32;
    /**
     * @example
     * nth(0) == lower
     * nth(length - 1) == upper
     *
     * @param nth 0-based
     * @return n-th element.
     */
    nth(nth: uint32): uint32;
    /**
     * Upper bound.
     */
    readonly upper: uint32;
    /**
     * @param other
     * @return {@link other } appended to this.
     */
    append(other: IntInterval): IntInterval;
    /**
     * @param index where split.
     *      0 < index < length
     * @return Right and left splits.
     */
    splitAt(index: uint32): [IntInterval, IntInterval];
    /**
     * @param other
     * @return this [order relation] other.
     */
    compare(other: IntInterval): IntervalOrdering;
}
