/*
    Copyright (C) 2018  Victorien Elvinger

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
import { assert } from "./assert";
export var UINT32_BOTTOM = 0;
export var UINT32_TOP = 4294967295; // 2^32 - 1
/**
 * @param n
 * @return Is {@link n} an uint32?
 */
export var isUint32 = function (n) {
    return Number.isSafeInteger(n) && UINT32_BOTTOM <= n && n <= UINT32_TOP;
};
/**
 * @param n
 * @return cast {@link n } as an uint32 (potential overflow).
 */
var asUint32 = function (n) { return n >>> 0; };
/**
 * @param l lower bound
 * @param excludedU excluded upper bound.
 * @return Random uint32 in [{@link l}, {@link excludedU}[
 *  Note that UINT32_TOP cannot be generated.
 */
export function nextRandomUint32(l, excludedU) {
    assert(function () { return isUint32(l); }, "l ∈ uint32");
    assert(function () { return isUint32(excludedU); }, "excludedU ∈ uint32");
    assert(function () { return l < excludedU; }, "l < excludedU");
    var randomPositiveFloat = (Math.random() * (excludedU - l)) + l;
    // Generate a random float number in [b1, b2[
    var result = randomPositiveFloat >>> 0;
    // Truncate the float in order to get an uint32
    assert(function () { return l <= result && result < excludedU; }, "result ∈ [l, excludedU[");
    return result;
}
/**
 * @param n1
 * @param n2
 * @return Order relation between {@link n1} and {@link n2}.
 */
export function compareUint32(n1, n2) {
    assert(function () { return isUint32(n1); }, "n1 ∈ uint32");
    assert(function () { return isUint32(n2); }, "n2 ∈ uint32");
    if (n1 < n2) {
        return -1 /* BEFORE */;
    }
    else if (n1 > n2) {
        return 1 /* AFTER */;
    }
    else {
        return 0 /* EQUAL */;
    }
}
/**
 * @param values
 * @return hash code of {@link values }.
 */
export var digestOf = function (values) {
    return values.reduce(function (acc, v) { return asUint32(asUint32(acc * 17) + v); });
};
export var absoluteSubstraction = function (a, b) {
    return (a < b) ? (b - a) : (a - b);
};
//# sourceMappingURL=number.js.map