/**
 * Possible relation between two elements in a totally ordered set.
 */
export declare const enum Ordering {
    BEFORE = -1,
    EQUAL = 0,
    AFTER = 1,
}
/**
 * Inversion of Ordering.
 *
 * @example
 * orderingInversion[Ordering.BEFORE] === Ordering.AFTER
 */
export declare const orderingInversion: Readonly<{
    [Ordering.BEFORE]: Ordering;
    [Ordering.EQUAL]: Ordering;
    [Ordering.AFTER]: Ordering;
}>;
/**
 * @example
 * lexCompareOrdering(Ordering.Equal, r2) == r2
 * lexCompareOrdering(Ordering.Before, _) == Ordering.Before
 * lexCompareOrdering(Ordering.After, _) == Ordering.After
 *
 * @param r1
 * @param r2
 * @return Lexicographic order between {@link r1} and {@link r2}.
 */
export declare const lexCompareOrdering: (r1: Ordering, r2: Ordering) => Ordering;
