import { Block, LengthBlock } from "../core/block";
import { BlockFactory } from "../core/blockfactory";
import { Concatenable } from "../core/concatenable";
import { Position } from "../core/position";
import { Insertion, Deletion } from "../core/localoperation";
import { uint32 } from "../core/number";
import { ReadonlyReplicatableList, ReplicatableList } from "../core/replicablelist";
import { Sentinel } from "./replicablelinkedlistcell";
export declare class ReadonlyReplicableLinkedList<P extends Position<P>, E extends Concatenable<E>> implements ReadonlyReplicatableList<P, E> {
    /**
     * New empty list.
     */
    constructor();
    /**
     * Map replica to their last observed seq.
     */
    protected readonly versionVector: {
        [replica: string]: uint32 | undefined;
    };
    /**
     * Chain entry.
     */
    readonly sentinel: Sentinel<P, E>;
    /** @Override */
    length: uint32;
    /** @Override */
    concatenated(prefix: E): E;
    /** @Override */
    readonly structuralDigest: uint32;
    /**
     * [Mutation]
     * Insert without updating and checking the version vector.
     * Do not use this function. It is exposed for testability purpose.
     *
     * @param block block to insert.
     * @return Performed modifications in terms of local insertions.
     *  The n+1 -th insertion depends on the effect of the n -th insertion.
     */
    insert(block: Block<P, E>): Insertion<E>[];
    /**
     * [Mutation]
     * Delete without updating and checking the version vector.
     * Do not use this function. It is exposed for testability purpose.
     *
     * @param block block to remove.
     * @return Performed modifications in terms of local deletions.
     *  The n+1 -th deletion depends on the n -th deletion.
     */
    remove(block: LengthBlock<P>): Deletion[];
    /** @Override */
    applyDelta(delta: LengthBlock<P>): Deletion[];
    applyDelta(delta: Block<P, E>): Insertion<E>[];
}
export declare class ReplicableLinkedList<P extends Position<P>, E extends Concatenable<E>> extends ReadonlyReplicableLinkedList<P, E> implements ReplicatableList<P, E> {
    protected factory: BlockFactory<P>;
    /**
     * @param factory strategy of block generation.
     * @param items items to insert.
     */
    constructor(factory: BlockFactory<P>, items: E);
    /** @Override */
    insertAt(index: uint32, items: E): Block<P, E>;
    /** @Override */
    removeAt(index: uint32, length: uint32): LengthBlock<P>[];
}
