"use strict";
/*
    Copyright (C) 2018  Victorien Elvinger

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var assert_1 = require("./assert");
var block_1 = require("./block");
var anchor_1 = require("./anchor");
/**
 * Factory of blocks.
 * Implementations can implement different strategies of generation.
 */
var BlockFactory = /** @class */ (function () {
    /**
     * @param posBounds bottom and top positions.
     */
    function BlockFactory(posBounds) {
        this.posBounds = posBounds;
        this.topAnchor = new anchor_1.Anchor(posBounds.TOP);
    }
    /**
     * @param items
     * @return New block with {@link items } as {@link Block#items }.
     */
    BlockFactory.prototype.from = function (items) {
        assert_1.assert(function () { return items.length > 0; }, "items.length > 0");
        var _a = __read(this.posBetween(this.posBounds.BOTTOM, items.length, this.posBounds.TOP), 2), pos = _a[0], factory = _a[1];
        return [new block_1.Block(pos, items), factory];
    };
    /**
     * @param l
     * @param items
     * @return New block after {@link l } with {@link items } as
     *  {@link Block#items }.
     */
    BlockFactory.prototype.after = function (l, items) {
        assert_1.assert(function () { return items.length > 0; }, "items.length > 0");
        var _a = __read(this.posBetween(l.upperPosition, items.length, this.posBounds.TOP), 2), pos = _a[0], factory = _a[1];
        return [new block_1.Block(pos, items), factory];
    };
    /**
     * @param items
     * @param u
     * @return New block before {@link u } with {@link items } as
     *  {@link Block#items }.
     */
    BlockFactory.prototype.before = function (items, u) {
        assert_1.assert(function () { return items.length > 0; }, "items.length > 0");
        var _a = __read(this.posBetween(this.posBounds.BOTTOM, items.length, u.lowerPosition), 2), pos = _a[0], factory = _a[1];
        return [new block_1.Block(pos, items), factory];
    };
    /**
     * @param l
     * @param items
     * @param u
     * @return New block after {@link l } and before {@link u }
     *  with {@link items } as {@link Block#items }.
     */
    BlockFactory.prototype.between = function (l, items, u) {
        assert_1.assert(function () { return items.length > 0; }, "items.length > 0");
        assert_1.heavyAssert(function () { return l.compare(u) <= -4 /* PREPENDABLE */; }, "l < u");
        var _a = __read(this.posBetween(l.upperPosition, items.length, u.lowerPosition), 2), pos = _a[0], factory = _a[1];
        return [new block_1.Block(pos, items), factory];
    };
    return BlockFactory;
}());
exports.BlockFactory = BlockFactory;
//# sourceMappingURL=blockfactory.js.map