"use strict";
/*
    Copyright (C) 2018  Victorien Elvinger

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
Object.defineProperty(exports, "__esModule", { value: true });
var assert_1 = require("./assert");
var number_1 = require("./number");
/**
 * Model an insertion in a single-user list.
 */
var Insertion = /** @class */ (function () {
    /**
     * @param index {@link Insertion#index}
     * @param items {@link Insertion#items}
     */
    function Insertion(index, items) {
        assert_1.assert(function () { return number_1.isUint32(index); }, "index ∈ uint32");
        assert_1.assert(function () { return items.length > 0; }, "length ∈ uint32");
        this.index = index;
        this.items = items;
    }
    Object.defineProperty(Insertion.prototype, "length", {
        /**
         * Number of inserted elements.
         */
        get: function () {
            return this.items.length;
        },
        enumerable: true,
        configurable: true
    });
    return Insertion;
}());
exports.Insertion = Insertion;
/**
 * Model a removal in a single-user list.
 */
var Deletion = /** @class */ (function () {
    /**
     * @param index Insertion#index
     * @param items Insertion#length
     */
    function Deletion(index, length) {
        assert_1.assert(function () { return number_1.isUint32(index); }, "index ∈ uint32");
        assert_1.assert(function () { return number_1.isUint32(length); }, "length ∈ uint32");
        assert_1.assert(function () { return length > 0; }, "length > 0");
        this.index = index;
        this.length = length;
    }
    return Deletion;
}());
exports.Deletion = Deletion;
//# sourceMappingURL=localoperation.js.map