"use strict";
/*
    Copyright (C) 2018  Victorien Elvinger

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
Object.defineProperty(exports, "__esModule", { value: true });
var assert_1 = require("./assert");
exports.UINT32_BOTTOM = 0;
exports.UINT32_TOP = 4294967295; // 2^32 - 1
/**
 * @param n
 * @return Is {@link n} an uint32?
 */
exports.isUint32 = function (n) {
    return Number.isSafeInteger(n) && exports.UINT32_BOTTOM <= n && n <= exports.UINT32_TOP;
};
/**
 * @param n
 * @return cast {@link n } as an uint32 (potential overflow).
 */
var asUint32 = function (n) { return n >>> 0; };
/**
 * @param l lower bound
 * @param excludedU excluded upper bound.
 * @return Random uint32 in [{@link l}, {@link excludedU}[
 *  Note that UINT32_TOP cannot be generated.
 */
function nextRandomUint32(l, excludedU) {
    assert_1.assert(function () { return exports.isUint32(l); }, "l ∈ uint32");
    assert_1.assert(function () { return exports.isUint32(excludedU); }, "excludedU ∈ uint32");
    assert_1.assert(function () { return l < excludedU; }, "l < excludedU");
    var randomPositiveFloat = (Math.random() * (excludedU - l)) + l;
    // Generate a random float number in [b1, b2[
    var result = randomPositiveFloat >>> 0;
    // Truncate the float in order to get an uint32
    assert_1.assert(function () { return l <= result && result < excludedU; }, "result ∈ [l, excludedU[");
    return result;
}
exports.nextRandomUint32 = nextRandomUint32;
/**
 * @param n1
 * @param n2
 * @return Order relation between {@link n1} and {@link n2}.
 */
function compareUint32(n1, n2) {
    assert_1.assert(function () { return exports.isUint32(n1); }, "n1 ∈ uint32");
    assert_1.assert(function () { return exports.isUint32(n2); }, "n2 ∈ uint32");
    if (n1 < n2) {
        return -1 /* BEFORE */;
    }
    else if (n1 > n2) {
        return 1 /* AFTER */;
    }
    else {
        return 0 /* EQUAL */;
    }
}
exports.compareUint32 = compareUint32;
/**
 * @param values
 * @return hash code of {@link values }.
 */
exports.digestOf = function (values) {
    return values.reduce(function (acc, v) { return asUint32(asUint32(acc * 17) + v); });
};
exports.absoluteSubstraction = function (a, b) {
    return (a < b) ? (b - a) : (a - b);
};
//# sourceMappingURL=number.js.map