"use strict";
/*
    Copyright (C) 2018  Victorien Elvinger

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
Object.defineProperty(exports, "__esModule", { value: true });
var assert_1 = require("./assert");
/**
 * Inversion of Ordering.
 *
 * @example
 * orderingInversion[Ordering.BEFORE] === Ordering.AFTER
 */
exports.orderingInversion = assert_1.readonly((_a = {},
    _a[-1 /* BEFORE */] = 1 /* AFTER */,
    _a[0 /* EQUAL */] = 0 /* EQUAL */,
    _a[1 /* AFTER */] = -1 /* BEFORE */,
    _a));
/**
 * @example
 * lexCompareOrdering(Ordering.Equal, r2) == r2
 * lexCompareOrdering(Ordering.Before, _) == Ordering.Before
 * lexCompareOrdering(Ordering.After, _) == Ordering.After
 *
 * @param r1
 * @param r2
 * @return Lexicographic order between {@link r1} and {@link r2}.
 */
exports.lexCompareOrdering = function (r1, r2) {
    return (r1 !== 0 /* EQUAL */) ? r1 : r2;
};
var _a;
//# sourceMappingURL=ordering.js.map