"use strict";
/*
    Copyright (C) 2018  Victorien Elvinger

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var assert_1 = require("../core/assert");
var number_1 = require("../core/number");
var replicablelinkedlistcell_1 = require("./replicablelinkedlistcell");
var ReadonlyReplicableLinkedList = /** @class */ (function () {
    /**
     * New empty list.
     */
    function ReadonlyReplicableLinkedList() {
        this.sentinel = new replicablelinkedlistcell_1.Sentinel();
        this.length = 0;
        this.versionVector = {}; //Object.create(null) is unsafe in TS
    }
    /** @Override */
    ReadonlyReplicableLinkedList.prototype.concatenated = function (prefix) {
        return this.sentinel.reduceBlock(function (acc, b) { return acc.concat(b.items); }, prefix);
    };
    Object.defineProperty(ReadonlyReplicableLinkedList.prototype, "structuralDigest", {
        /** @Override */
        get: function () {
            return this.sentinel.reduceBlock(function (acc, b) { return number_1.digestOf([acc, b.structuralDigest]); }, 0);
        },
        enumerable: true,
        configurable: true
    });
    // Modification
    /**
     * [Mutation]
     * Insert without updating and checking the version vector.
     * Do not use this function. It is exposed for testability purpose.
     *
     * @param block block to insert.
     * @return Performed modifications in terms of local insertions.
     *  The n+1 -th insertion depends on the effect of the n -th insertion.
     */
    ReadonlyReplicableLinkedList.prototype.insert = function (block) {
        var result = this.sentinel.insert(block, 0);
        this.length = result.reduce(function (acc, v) { return acc + v.length; }, this.length);
        return result;
    };
    /**
     * [Mutation]
     * Delete without updating and checking the version vector.
     * Do not use this function. It is exposed for testability purpose.
     *
     * @param block block to remove.
     * @return Performed modifications in terms of local deletions.
     *  The n+1 -th deletion depends on the n -th deletion.
     */
    ReadonlyReplicableLinkedList.prototype.remove = function (block) {
        var result = this.sentinel.remove(block, 0);
        this.length = result.reduce(function (acc, v) { return acc - v.length; }, this.length);
        return result;
    };
    ReadonlyReplicableLinkedList.prototype.applyDelta = function (delta) {
        var deltaSeqs = delta.seqs;
        //const lastSeq = this.versionVector.get(delta.replica)
        var lastSeq = this.versionVector[delta.replica];
        if (delta.isLengthBlock()) {
            if (lastSeq !== undefined) {
                this.versionVector[delta.replica] = Math.max(lastSeq, deltaSeqs.upper);
            }
            else {
                this.versionVector[delta.replica] = deltaSeqs.upper;
            }
            return this.remove(delta);
        }
        else {
            if (lastSeq === undefined) {
                this.versionVector[delta.replica] = deltaSeqs.upper;
                return this.insert(delta);
            }
            else if (deltaSeqs.lower > (lastSeq + 1)) {
                assert_1.assert(function () { return false; }, "FIFO violation");
                return [];
            }
            else if (deltaSeqs.upper <= lastSeq) {
                return [];
            }
            else {
                var remaining = delta.rightSplitAt(lastSeq + 1 - deltaSeqs.lower);
                this.versionVector[delta.replica] = deltaSeqs.upper;
                return this.insert(remaining);
            }
        }
    };
    return ReadonlyReplicableLinkedList;
}());
exports.ReadonlyReplicableLinkedList = ReadonlyReplicableLinkedList;
var ReplicableLinkedList = /** @class */ (function (_super) {
    __extends(ReplicableLinkedList, _super);
    /**
     * @param factory strategy of block generation.
     * @param items items to insert.
     */
    function ReplicableLinkedList(factory, items) {
        var _this = _super.call(this) || this;
        _this.factory = factory;
        _this.factory = factory;
        if (items.length > 0) {
            _this.insertAt(0, items);
        }
        else if (factory.seq > 0) {
            _this.versionVector[factory.replica] = factory.seq - 1;
        }
        return _this;
    }
    // Modification
    /** @Override */
    ReplicableLinkedList.prototype.insertAt = function (index, items) {
        var _this = this;
        assert_1.assert(function () { return number_1.isUint32(index); }, "index ∈ uint32");
        assert_1.assert(function () { return index <= _this.length; }, "valid index");
        assert_1.assert(function () { return items.length > 0; }, "items is not empty");
        assert_1.assert(function () { return number_1.isUint32(_this.length + items.length); }, "(items.length + this.length) ∈ uint32");
        var _a = __read(this.sentinel.insertAt(index, items, this.factory), 2), result = _a[0], newFactory = _a[1];
        this.factory = newFactory;
        this.length = this.length + items.length;
        this.versionVector[result.replica] = result.seqs.upper;
        return result;
    };
    /** @Override */
    ReplicableLinkedList.prototype.removeAt = function (index, length) {
        var _this = this;
        assert_1.assert(function () { return number_1.isUint32(index); }, "index ∈ uint32");
        assert_1.assert(function () { return number_1.isUint32(length); }, "length ∈ uint32");
        assert_1.assert(function () { return number_1.isUint32(index + length); }, "(index + length) ∈ uint32");
        assert_1.assert(function () { return length > 0; }, "length > 0");
        assert_1.assert(function () { return (index + length) <= _this.length; }, "(index + length) <= this.length");
        this.length = this.length - length;
        return this.sentinel.removeAt(index, length);
    };
    return ReplicableLinkedList;
}(ReadonlyReplicableLinkedList));
exports.ReplicableLinkedList = ReplicableLinkedList;
//# sourceMappingURL=replicablelinkedlist.js.map