"use strict";
/*
    Copyright (C) 2018  Victorien Elvinger

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spread = (this && this.__spread) || function () {
    for (var ar = [], i = 0; i < arguments.length; i++) ar = ar.concat(__read(arguments[i]));
    return ar;
};
Object.defineProperty(exports, "__esModule", { value: true });
var assert_1 = require("../core/assert");
var localoperation_1 = require("../core/localoperation");
var number_1 = require("../core/number");
var Linkable = /** @class */ (function () {
    /**
     * @param right {@link Linkable#right }
     */
    function Linkable(right) {
        this.right = right;
    }
    Linkable.prototype.reduceBlock = function (f, prefix) {
        if (this.right === undefined) {
            return prefix;
        }
        else {
            var rBlock = this.right.block;
            return this.right.reduceBlock(f, f(prefix, rBlock)); // Tail recursion
        }
    };
    // Modification
    /**
     * [Mutation]
     * Insert {@link rblock } to right and preserve the chain.
     *
     * @param rblock block to insert.
     * @return inserted cell with {@link rblock } as content.
     */
    Linkable.prototype.insertRight = function (rblock) {
        this.right = new Cell(rblock, this.right);
        return this.right;
    };
    /**
     * [Mutation]
     * Insert the parts of {@link iBlock } which are not already inserted.
     *
     * @param iBlock block to insert.
     * @param index 0-based index
     * @return Performed modifications in terms of local insertions.
     *  The n+1 -th insertion depends on the effect of the n -th insertion.
     */
    Linkable.prototype.insert = function (iBlock, index) {
        assert_1.assert(function () { return number_1.isUint32(index); }, "index ∈ uint32");
        if (this.right === undefined) {
            this.insertRight(iBlock);
            return [new localoperation_1.Insertion(index, iBlock.items)];
        }
        else {
            var rBlock = this.right.block;
            switch (rBlock.compare(iBlock)) {
                case -5 /* BEFORE */:
                    return this.right.insert(iBlock, index + rBlock.length);
                case -4 /* PREPENDABLE */: {
                    var insertions = this.right.insert(iBlock, index + rBlock.length);
                    // The right block of rBlock could be a splitting block
                    this.right = this.right.right;
                    this.insert(rBlock, 0); // handle prepend
                    return insertions;
                }
                case 5 /* AFTER */:
                    this.insertRight(iBlock);
                    return [new localoperation_1.Insertion(index, iBlock.items)];
                case 4 /* APPENDABLE */:
                    this.right = this.right.right;
                    this.insertRight(iBlock.append(rBlock));
                    return [new localoperation_1.Insertion(index, iBlock.items)];
                case 2 /* SPLITTING */: {
                    var _a = __read(iBlock.splitWith(rBlock), 2), lSplit = _a[0], rSplit = _a[1];
                    var rInsertions = this.right.insert(rSplit, index + lSplit.length + rBlock.length);
                    this.insertRight(lSplit);
                    return __spread([new localoperation_1.Insertion(index, lSplit.items)], rInsertions);
                }
                case -2 /* SPLITTED_BY */: {
                    var _b = __read(rBlock.splitWith(iBlock), 2), lSplit = _b[0], rSplit = _b[1];
                    this.right = this.right.right;
                    this.insertRight(lSplit)
                        .insertRight(iBlock)
                        .insertRight(rSplit);
                    return [new localoperation_1.Insertion(index + lSplit.length, iBlock.items)];
                }
                case -1 /* INCLUDED_BY */: {
                    var _c = __read(iBlock.remove(rBlock), 2), lRemaning = _c[0], rRemaining = _c[1];
                    var insertions = [];
                    if (lRemaning !== undefined) {
                        this.right = this.right.right;
                        insertions = [new localoperation_1.Insertion(index, lRemaning.items)];
                        this.insertRight(lRemaning.append(rBlock));
                    }
                    if (rRemaining !== undefined) {
                        insertions = __spread(insertions, this.insert(rRemaining, index));
                    }
                    return insertions;
                }
                case -3 /* OVERLAPPING_BEFORE */: {
                    var appendable = iBlock.appendable(rBlock);
                    return this.insert(appendable, index); // Tail recursion
                }
                case 3 /* OVERLAPPING_AFTER */: {
                    var prependable = iBlock.prependable(rBlock);
                    this.right = this.right.right;
                    this.insertRight(prependable.append(rBlock));
                    return [new localoperation_1.Insertion(index, prependable.items)];
                }
                default://case BlockOrdering.Including, BlockOrdering.Equal:
                    return []; // already inserted
            }
        }
    };
    /**
     * [Mutation]
     * Insert {@link items } at {@link index}.
     *
     * @param index 0-based index; Where to insert.
     * @param items elements to insert.
     * @param factory strategy of block generation.
     * @return Delta which represents the insertion.
     */
    Linkable.prototype.insertAt = function (index, items, factory) {
        var _this = this;
        assert_1.assert(function () { return number_1.isUint32(index); }, "index ∈ uint32");
        assert_1.assert(function () { return items.length > 0; }, "items.length > 0");
        var generated;
        if (this.right === undefined) {
            if (this instanceof Cell) {
                generated = factory.after(this.block, items);
            }
            else {
                generated = factory.from(items);
            }
            this.insertRight(generated[0]);
        }
        else {
            var rBlock = this.right.block;
            if (index > rBlock.length) {
                generated = this.right.insertAt(index - rBlock.length, items, factory);
            }
            else if (index === rBlock.length) {
                generated = (this.right.right !== undefined)
                    ? factory.between(rBlock, items, this.right.right.block)
                    : factory.after(rBlock, items);
                this.insert(generated[0], 0); // handle append and prepend
            }
            else if (index === 0) {
                assert_1.assert(function () { return _this instanceof Sentinel; }, "only Sentinel can insert at index 0");
                generated = factory.before(items, rBlock);
                this.insert(generated[0], 0); // handle prepend
            }
            else {
                var _a = __read(rBlock.splitAt(index), 2), lSplit = _a[0], rSplit = _a[1];
                generated = factory.between(lSplit, items, rSplit);
                this.right = this.right.right; // remove right
                this.insertRight(lSplit)
                    .insertRight(generated[0])
                    .insertRight(rSplit);
            }
        }
        return generated;
    };
    /**
     * [Mutation]
     * Remove a number of {@link length } elements from {@link index }
     *
     * @param index 0-based index.
     * @param length Number of elements to remove.
     * @return Delta which represents the deletion.
     */
    Linkable.prototype.removeAt = function (index, length) {
        assert_1.assert(function () { return number_1.isUint32(index); }, "index ∈ uint32");
        assert_1.assert(function () { return number_1.isUint32(length); }, "length ∈ uint32");
        assert_1.assert(function () { return length > 0; }, "length > 0");
        if (this.right !== undefined) {
            var rBlock = this.right.block;
            if (index > rBlock.length) {
                return this.right.removeAt(index - rBlock.length, length);
            }
            else if (index === rBlock.length) {
                var removeds = this.right.removeAt(0, length);
                this.right = this.right.right;
                this.insert(rBlock, 0); // handle append
                return removeds;
            }
            else if (index !== 0) {
                var _a = __read(rBlock.splitAt(index), 2), lSplit = _a[0], rSplit = _a[1];
                this.right = this.right.right;
                var newRight = this.insertRight(lSplit);
                newRight.insertRight(rSplit);
                return newRight.removeAt(index - lSplit.length, length);
            }
            else {
                if (rBlock.length === length) {
                    this.right = this.right.right;
                    return [rBlock.toLengthBlock()];
                }
                else if (rBlock.length < length) {
                    this.right = this.right.right;
                    var removeds = this.removeAt(index, length - rBlock.length);
                    return __spread([rBlock.toLengthBlock()], removeds);
                }
                else {
                    var _b = __read(rBlock.splitAt(length), 2), lSplit = _b[0], rSplit = _b[1];
                    this.right = this.right.right;
                    this.insertRight(rSplit);
                    return [lSplit.toLengthBlock()];
                }
            }
        }
        else {
            return [];
        }
    };
    /**
     * [Mutation]
     * Remove {@link dBlock }.
     * @param dBlock block to remove.
     * @param index index of the current block in the chain.
     * @return Performed modifications in terms of local deletions.
     *  The n+1 -th deletion depends on the n -th deletion.
     */
    Linkable.prototype.remove = function (dBlock, index) {
        assert_1.assert(function () { return number_1.isUint32(index); }, "index ∈ uint32");
        if (this.right !== undefined) {
            var rBlock = this.right.block;
            switch (rBlock.compare(dBlock)) {
                case 2 /* SPLITTING */: // fall-through
                case -5 /* BEFORE */: {
                    var removals = this.right.remove(dBlock, index + rBlock.length);
                    var rIndex = index + rBlock.length;
                    if (removals.length > 0 && removals[0].index === rIndex) {
                        this.right = this.right.right;
                        this.insert(rBlock, 0); // handle append
                    }
                    return removals;
                }
                case -4 /* PREPENDABLE */:
                    return this.right.remove(dBlock, index + rBlock.length); // tail recursion
                case 1 /* INCLUDING */: {
                    var removed = rBlock.intersection(dBlock);
                    var _a = __read(rBlock.remove(dBlock), 2), lRemaning = _a[0], rRemaning = _a[1];
                    this.right = this.right.right;
                    if (rRemaning !== undefined) {
                        this.insertRight(rRemaning);
                    }
                    var removalIndex = index;
                    if (lRemaning !== undefined) {
                        this.insertRight(lRemaning);
                        removalIndex = removalIndex + lRemaning.length;
                    }
                    return [new localoperation_1.Deletion(removalIndex, dBlock.length)];
                }
                case 0 /* EQUAL */:
                    this.right = this.right.right;
                    return [new localoperation_1.Deletion(index, rBlock.length)];
                case -1 /* INCLUDED_BY */: {
                    // Append of the current block is handled in the first switch-case
                    this.right = this.right.right;
                    var rRemovals = this.remove(dBlock, index);
                    return __spread([new localoperation_1.Deletion(index, rBlock.length)], rRemovals);
                }
                case -3 /* OVERLAPPING_BEFORE */: {
                    var removed = rBlock.intersection(dBlock);
                    var remaining = rBlock.prependable(dBlock);
                    //const [remaining, ] = rBlock.remove(block) as [undefined, Block<P, E>]
                    //remaining = remaining as Block<P, E>
                    this.right = this.right.right;
                    this.insertRight(remaining);
                    var removals = this.remove(dBlock, index + remaining.length);
                    return [new localoperation_1.Deletion(index + remaining.length, removed.length)];
                }
                case 3 /* OVERLAPPING_AFTER */: {
                    var removed = rBlock.intersection(dBlock);
                    var remaining = rBlock.appendable(dBlock);
                    //const [, remaining] = rBlock.remove(block) as [undefined, Block<P, E>]
                    this.right = this.right.right;
                    this.insertRight(remaining);
                    return [new localoperation_1.Deletion(index, removed.length)];
                }
                //case BlockOrdering.After:
                //case BlockOrdering.Appendable:
                //case BlockOrdering.SplittedBy:
                default:
            }
        }
        return []; // already removed or not already inserted
    };
    return Linkable;
}());
exports.Linkable = Linkable;
var Sentinel = /** @class */ (function (_super) {
    __extends(Sentinel, _super);
    function Sentinel() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    return Sentinel;
}(Linkable));
exports.Sentinel = Sentinel;
var Cell = /** @class */ (function (_super) {
    __extends(Cell, _super);
    /**
     * @param block {@link Cell#block }
     * @param right {@link Linkable#right }
     */
    function Cell(block, right) {
        var _this = this;
        assert_1.heavyAssert(function () { return (right === undefined ||
            block.compare(right.block) === -5 /* BEFORE */); }, "block < right.block");
        _this = _super.call(this, right) || this;
        _this.block = block;
        return _this;
    }
    return Cell;
}(Linkable));
exports.Cell = Cell;
//# sourceMappingURL=replicablelinkedlistcell.js.map