"use strict";
/*
    Copyright (C) 2018  Victorien Elvinger

    This Source Code Form is subject to the terms of the Mozilla Public
    License, v. 2.0. If a copy of the MPL was not distributed with this
    file, You can obtain one at http://mozilla.org/MPL/2.0/.
*/
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var assert_1 = require("../core/assert");
var simpleposition_1 = require("./simpleposition");
var simplepositionpart_1 = require("./simplepositionpart");
var number_1 = require("../core/number");
var blockfactory_1 = require("../core/blockfactory");
/**
 * Factory of block with {@link SimplePosition } as implementation of
 * {@link Position}.
 * The strategy of generation is to genearte random positions between two
 * defined positions.
 */
var SimpleBlockFactory = /** @class */ (function (_super) {
    __extends(SimpleBlockFactory, _super);
    /**
     * @param posBounds {@link SimpleBlockFactory#replica }
     * @param seq {@link SimpleBlockFactory#seq }
     */
    function SimpleBlockFactory(replica, seq) {
        var _this = this;
        assert_1.assert(function () { return number_1.isUint32(replica); }, "replica ∈ uint32");
        assert_1.assert(function () { return replica !== number_1.UINT32_TOP; }, "replica != UINT32_TOP. This is reserved for BOTTOM and TOP positions.");
        _this = _super.call(this, simpleposition_1.SimplePosition) || this;
        assert_1.assert(function () { return number_1.isUint32(seq); }, "seq ∈ uint32");
        _this.replica = replica;
        _this.seq = seq;
        return _this;
    }
    /**
     * @param replica {@link SimpleBlockFactory#replica }.
     * @return New factory with 0 as {@link SimpleBlockFactory#seq }.
     */
    SimpleBlockFactory.from = function (replica) {
        assert_1.assert(function () { return number_1.isUint32(replica); }, "replica ∈ uint32");
        assert_1.assert(function () { return replica !== number_1.UINT32_TOP; }, "replica != UINT32_TOP. This is reserved for BOTTOM and TOP positions.");
        return new SimpleBlockFactory(replica, 0);
    };
    // Derivation
    /**
     * @param by
     * @return Same factory, but with {@link SimpleBlockFactory#seq }
     * increased by {@link by }
     */
    SimpleBlockFactory.prototype.increasedSeq = function (by) {
        var _this = this;
        assert_1.assert(function () { return number_1.isUint32(by); }, "by ∈ uint32");
        assert_1.assert(function () { return number_1.isUint32(_this.seq + by); }, "no overflow");
        return new SimpleBlockFactory(this.replica, this.seq + by);
    };
    // Impl
    /** @override */
    SimpleBlockFactory.prototype.posBetween = function (l, length, u) {
        var _this = this;
        assert_1.heavyAssert(function () { return l.compare(u) === -1 /* BEFORE */; }, "l < u");
        assert_1.assert(function () { return number_1.isUint32(length); }, "length ∈ uint32");
        assert_1.assert(function () { return length > 0; }, "length is strictly positive");
        assert_1.assert(function () { return number_1.isUint32(_this.seq + length); }, "no overflow");
        var factory = this.increasedSeq(length);
        var lastLIndex = l.depth - 1;
        if (l.replica === this.replica && (l.seq + 1) === this.seq) {
            // Appendable
            return [l.intSuccessor(1), factory];
        }
        else {
            var seqL = infiniteSequence(l.parts, simplepositionpart_1.SimplePositionPart.BOTTOM);
            var seqU = infiniteSequence(u.parts, simplepositionpart_1.SimplePositionPart.TOP);
            var parts = [];
            var partL = seqL.next().value;
            var partU = seqU.next().value;
            while ((partU.priority - partL.priority) < 2) {
                // Cannot insert a new part between partL and partU
                if (partU.replica === partL.replica) {
                    // Split
                    parts.push(partL);
                }
                else {
                    parts.push(partL.withSeq(number_1.UINT32_TOP));
                }
                partL = seqL.next().value;
                partU = seqU.next().value;
            }
            var priority = number_1.nextRandomUint32(partL.priority + 1, partU.priority);
            // priority ∈ ]tuple1.priority, tuple2.priority[
            // tuple1.priority exclusion ensures a dense set
            parts.push(simplepositionpart_1.SimplePositionPart.from(priority, this.replica, this.seq));
            return [simpleposition_1.SimplePosition.from(parts), factory];
        }
    };
    return SimpleBlockFactory;
}(blockfactory_1.BlockFactory));
exports.SimpleBlockFactory = SimpleBlockFactory;
/**
 * @param values
 * @param defaultValue value to emit once {@link values } were emitted.
 * @return Infinite stream of values.
 */
function infiniteSequence(values, defaultValue) {
    var values_1, values_1_1, v, e_1_1, e_1, _a;
    return __generator(this, function (_b) {
        switch (_b.label) {
            case 0:
                _b.trys.push([0, 5, 6, 7]);
                values_1 = __values(values), values_1_1 = values_1.next();
                _b.label = 1;
            case 1:
                if (!!values_1_1.done) return [3 /*break*/, 4];
                v = values_1_1.value;
                return [4 /*yield*/, v];
            case 2:
                _b.sent();
                _b.label = 3;
            case 3:
                values_1_1 = values_1.next();
                return [3 /*break*/, 1];
            case 4: return [3 /*break*/, 7];
            case 5:
                e_1_1 = _b.sent();
                e_1 = { error: e_1_1 };
                return [3 /*break*/, 7];
            case 6:
                try {
                    if (values_1_1 && !values_1_1.done && (_a = values_1.return)) _a.call(values_1);
                }
                finally { if (e_1) throw e_1.error; }
                return [7 /*endfinally*/];
            case 7:
                if (!true) return [3 /*break*/, 9];
                return [4 /*yield*/, defaultValue];
            case 8:
                _b.sent();
                return [3 /*break*/, 7];
            case 9: return [2 /*return*/];
        }
    });
}
//# sourceMappingURL=simpleblockfactory.js.map